##
# A very simple bounded double-ended queue/stack used for caching a list of values. Overrides a few
# methods from Array so that if more than a certain number of elements are added, entries at the
# back of the array will be popped off
class BoundedQueue < Array

  private
  alias_method :push_internal, :push
  alias_method :unshift_internal, :unshift

  public

  ##
  # Initialize a queue with a set length. The backing array is not resized to match this size.
  def initialize(max_size)
    super()
    @max_size = max_size
  end

  ##
  # Pushes an element onto the front of the "queue", removing the last element in the queue if
  # necessary.
  def push(element)
    pop if size + 1 >= @max_size
    push_internal(element)
  end

  ##
  # Pushes an element onto the front of the "stack", removing the first element in the stack if
  # necessary.
  def unshift(element)
    shift if size + 1 >= @max_size
    unshift_internal(element)
  end

end