class Patch
  attr_reader :resources, :area, :num_bands

  def initialize(resources, conf)
    stealables = [:area,:max_bands]
    stealables.each{|s|self.instance_variable_set("@#{s.to_s}".to_sym, conf[s.to_s])}
    @resources = resources.map{|type,cc| Object.const_get(type).new(cc, area)}

    @num_bands = 0
  end

  # serves a join attempt; returns success status
  def join
    if joinable?
      @num_bands += 1
      return true
    end
    false # Patch full, no room for more
  end

  def joinable?
    @num_bands < @max_bands or @max_bands == 0
  end

  def leave
    if @num_bands <= 0
      warn "Error: a band attempted to leave a patch with no bands"
      return
    end
    @num_bands -= 1
  end

  def do_time_step
    @resources.each { |r| r.grow }
  end

  def depleted?
    @resources.all? { |r| r.depleted? }
  end

  # does it have a viable population for the specified resource type?
  def viable?(resource)
    found = @resources.find { |r| r.name == resource }
    return found.viable? if found
    false
  end

  def to_s
    resources = "[" << @resources.map(&:to_s).reduce { |m, o| "#{m}, #{o}" } << "]" # Try that in Java
    "Patch (id:#{object_id}) { Bands: #{@num_inhabitants}/#{@max_bands} | Resources:#{resources} }"
  end

end
# vim: et:ts=2:sw=2
