##
# A set of static utility methods

##
# Returns true with a probability of 'percent'
def chance(percent)
  rand(100) <= percent
end

##
# Intern the string keys to a hash
def hash_intern(h)
  h.inject({}) { |memo, (k, v)| memo[k.to_sym] = v; memo }
end

##
# Add some handy sorting methods to the Array class

class Array
  ##
  # Performs a radix sort of the array using a given base (default 10).
  def radix_sort(base=10)
    ary = dup
    rounds = (Math.log(self.max.abs)/Math.log(base)).ceil
    rounds.times do |i|
      buckets = Hash.new {|h,k| h[k] = []}
      ary.each do |n|
        digit = (n/base**i) % base
        digit = digit + base unless n<0
        buckets[digit] << n
      end
      ary = buckets.values_at(*(0..2*base)).compact.flatten
      p [i, ary] if $DEBUG
    end
    ary
  end

  ##
  # Destructive version of radix_sort
  def radix_sort!(base=10)
    replace radix_sort(base)
  end

end
# vim: et:ts=2:sw=2
